@@ -9,17 +9,14 @@ import warnings
 
 from vine import ensure_promise
 
-from . import __version__
-from . import sasl
-from . import spec
+from . import __version__, sasl, spec
 from .abstract_channel import AbstractChannel
 from .channel import Channel
-from .exceptions import (
-    AMQPDeprecationWarning, ChannelError, ResourceError,
-    ConnectionForced, ConnectionError, error_for_code,
-    RecoverableConnectionError, RecoverableChannelError,
-)
-from .five import array, items, monotonic, range, values, string
+from .exceptions import (AMQPDeprecationWarning, ChannelError, ConnectionError,
+                         ConnectionForced, RecoverableChannelError,
+                         RecoverableConnectionError, ResourceError,
+                         error_for_code)
+from .five import array, items, monotonic, range, string, values
 from .method_framing import frame_handler, frame_writer
 from .transport import Transport
 
@@ -86,7 +83,13 @@ class Connection(AbstractChannel):
 
     Authentication can be controlled by passing one or more
     `amqp.sasl.SASL` instances as the `authentication` parameter, or
-    by using the userid and password parameters (for AMQPLAIN and PLAIN).
+    setting the `login_method` string to one of the supported methods:
+    'GSSAPI', 'EXTERNAL', 'AMQPLAIN', or 'PLAIN'.
+    Otherwise authentication will be performed using any supported method
+    preferred by the server. Userid and passwords apply to AMQPLAIN and
+    PLAIN authentication, whereas on GSSAPI only userid will be used as the
+    client name. For EXTERNAL authentication both userid and password are
+    ignored.
 
     The 'ssl' parameter may be simply True/False, or for Python >= 2.6
     a dictionary of options to pass to ssl.wrap_socket() such as
@@ -94,6 +97,10 @@ class Connection(AbstractChannel):
 
     The "socket_settings" parameter is a dictionary defining tcp
     settings which will be applied as socket options.
+
+    When "confirm_publish" is set to True, the channel is put to
+    confirm mode. In this mode, each published message is
+    confirmed using Publisher confirms RabbitMQ extention.
     """
 
     Channel = Channel
@@ -189,14 +196,31 @@ class Connection(AbstractChannel):
             if isinstance(authentication, sasl.SASL):
                 authentication = (authentication,)
             self.authentication = authentication
-        elif login_method is not None and login_response is not None:
-            self.authentication = (sasl.RAW(login_method, login_response),)
-        elif userid is not None and password is not None:
+        elif login_method is not None:
+            if login_method == 'GSSAPI':
+                auth = sasl.GSSAPI(userid)
+            elif login_method == 'EXTERNAL':
+                auth = sasl.EXTERNAL()
+            elif login_method == 'AMQPLAIN':
+                if userid is None or password is None:
+                    raise ValueError(
+                        "Must supply authentication or userid/password")
+                auth = sasl.AMQPLAIN(userid, password)
+            elif login_method == 'PLAIN':
+                if userid is None or password is None:
+                    raise ValueError(
+                        "Must supply authentication or userid/password")
+                auth = sasl.PLAIN(userid, password)
+            elif login_response is not None:
+                auth = sasl.RAW(login_method, login_response)
+            else:
+                raise ValueError("Invalid login method", login_method)
+            self.authentication = (auth,)
+        else:
             self.authentication = (sasl.GSSAPI(userid, fail_soft=True),
+                                   sasl.EXTERNAL(),
                                    sasl.AMQPLAIN(userid, password),
                                    sasl.PLAIN(userid, password))
-        else:
-            raise ValueError("Must supply authentication or userid/password")
 
         self.client_properties = dict(
             self.library_properties, **client_properties or {}
@@ -274,18 +298,23 @@ class Connection(AbstractChannel):
         #
         if self.connected:
             return callback() if callback else None
-        self.transport = self.Transport(
-            self.host, self.connect_timeout, self.ssl,
-            self.read_timeout, self.write_timeout,
-            socket_settings=self.socket_settings,
-        )
-        self.transport.connect()
-        self.on_inbound_frame = self.frame_handler_cls(
-            self, self.on_inbound_method)
-        self.frame_writer = self.frame_writer_cls(self, self.transport)
+        try:
+            self.transport = self.Transport(
+                self.host, self.connect_timeout, self.ssl,
+                self.read_timeout, self.write_timeout,
+                socket_settings=self.socket_settings,
+            )
+            self.transport.connect()
+            self.on_inbound_frame = self.frame_handler_cls(
+                self, self.on_inbound_method)
+            self.frame_writer = self.frame_writer_cls(self, self.transport)
 
-        while not self._handshake_complete:
-            self.drain_events(timeout=self.connect_timeout)
+            while not self._handshake_complete:
+                self.drain_events(timeout=self.connect_timeout)
+
+        except (OSError, IOError, SSLError):
+            self.collect()
+            raise
 
     def _warn_force_connect(self, attr):
         warnings.warn(AMQPDeprecationWarning(
@@ -539,11 +568,18 @@ class Connection(AbstractChannel):
             # already closed
             return
 
-        return self.send_method(
-            spec.Connection.Close, argsig,
-            (reply_code, reply_text, method_sig[0], method_sig[1]),
-            wait=spec.Connection.CloseOk,
-        )
+        try:
+            self.is_closing = True
+            return self.send_method(
+                spec.Connection.Close, argsig,
+                (reply_code, reply_text, method_sig[0], method_sig[1]),
+                wait=spec.Connection.CloseOk,
+            )
+        except (OSError, IOError, SSLError):
+            self.is_closing = False
+            # close connection
+            self.collect()
+            raise
 
     def _on_close(self, reply_code, reply_text, class_id, method_id):
         """Request a connection close.
